@@ -75,7 +75,7 @@ int ftp_chmod(FTPSTATE *peer, char *filename, int mode)
 int ftp_size(FTPSTATE *peer, char *filename)
 {
 	int filefd;
-	int size = 0;
+	off_t size = 0;
 	int asize;
 	struct stat statdata;
 	
@@ -114,7 +114,7 @@ int ftp_size(FTPSTATE *peer, char *filename)
 	}
 
 	if (filefd >= 0)
-		ftp_write(peer, FALSE, 213, "%d", size); 
+		ftp_write(peer, FALSE, 213, "%s", offt_tostr(size)); 
 	else
 		reporterror(peer, filename, errno);
 
@@ -159,12 +159,12 @@ void setdumptokens(FTPSTATE *peer, TOKENSET *ts)
 	tokenset_settoken(ts, 'L', strdupwrapper(peer->vserver->vhostname));
 	tokenset_settoken(ts, 'f', safe_snprintf("%d", peer->downloadedfiles));
 	tokenset_settoken(ts, 'F', safe_snprintf("%d", peer->uploadedfiles));
-	tokenset_settoken(ts, 'b', safe_snprintf("%d", peer->downloadedfilebytes));
-	tokenset_settoken(ts, 'B', safe_snprintf("%d", peer->uploadedfilebytes));
-	tokenset_settoken(ts, 'I', safe_snprintf("%d", peer->listdownloadedbytes));
+	tokenset_settoken(ts, 'b', safe_snprintf("%s", offt_tostr(peer->downloadedfilebytes)));
+	tokenset_settoken(ts, 'B', safe_snprintf("%s", offt_tostr(peer->uploadedfilebytes)));
+	tokenset_settoken(ts, 'I', safe_snprintf("%s", offt_tostr(peer->listdownloadedbytes)));
 	tokenset_settoken(ts, 'i', safe_snprintf("%d", peer->listconns));
-	tokenset_settoken(ts, 'D', safe_snprintf("%d", peer->listdownloadedbytes + peer->downloadedfilebytes));
-	tokenset_settoken(ts, 't', safe_snprintf("%d", peer->listdownloadedbytes + peer->downloadedfilebytes + peer->uploadedfilebytes));
+	tokenset_settoken(ts, 'D', safe_snprintf("%s", offt_tostr(peer->listdownloadedbytes + peer->downloadedfilebytes)));
+	tokenset_settoken(ts, 't', safe_snprintf("%s", offt_tostr(peer->listdownloadedbytes + peer->downloadedfilebytes + peer->uploadedfilebytes)));
 	tokenset_settoken(ts, 'c', safe_snprintf("%d", peer->listconns + peer->downloadedfiles + peer->uploadedfiles));
 	tokenset_settoken(ts, 'd', safe_snprintf("%d", peer->listconns + peer->downloadedfiles));
 	if (peer->ratioinfo)
@@ -683,9 +683,12 @@ int ftp_pass(FTPSTATE *peer, char *param)
 
 int ftp_rest(FTPSTATE *peer, char *param)
 {
-	int newpos, res;
-	res = sscanf(param, "%d", &newpos);
-	if ((res != 1) && (newpos > 0))
+	off_t newpos;
+	int res;
+
+	res = strto_offt(param, &newpos);
+	
+	if ((res != 0) || (newpos < 0))
 		return(ftp_write(peer, FALSE, 501, REPLY_INVALIDREST));
 	else
 	{
