@@ -225,15 +225,15 @@ typedef struct dataport
 {
 	int filefd;
 	int socketfd;
-	int pos;
-	int startpos;
-	int tsize;
+	off_t pos;
+	off_t startpos;
+	off_t tsize;
 	STRING *buffer;
 	LISTHANDLE *lhandle;
 	int trans_type;
 	int binary;
 	int passive;
-	int transbytes;
+	off_t transbytes;
 	LIMITER *download_limiter;
 	LIMITER *upload_limiter;
 } DATAPORT;
@@ -333,7 +333,7 @@ typedef struct ftpstate
 	int passiveport;	/* the fd of the listening passive port */
 	char *renameoldname;	/* the source file to rename */
 	int loggedin;		/* true if the user is logged in */
-	int restartpos;		/* the position to start a download or upload */
+	off_t restartpos;	/* the position to start a download or upload */
 	int remoteport;		/* the port number to connect to for data conns */
 	int binary;		/* true if binary mode is set */
 	int threadnum;		/* the number of the thread in ftpd terms (not pid) */
@@ -389,9 +389,9 @@ typedef struct ftpstate
 	/* Counters for stats */
 	int downloadedfiles;
 	int uploadedfiles;
-	int downloadedfilebytes;
-	int uploadedfilebytes;
-	int listdownloadedbytes;
+	off_t downloadedfilebytes;
+	off_t uploadedfilebytes;
+	off_t listdownloadedbytes;
 	int listconns;
 } FTPSTATE;
 
@@ -473,16 +473,6 @@ void select_takewrite(SELECTER *sel, int port);
 int select_do(SELECTER *sel, int *signum, int timeout);
 void select_shutdown(SELECTER *sel);
 
-/* string.h */
-
-STRING *string_new(void);
-void string_cat(STRING **s, char *catstr, int len);
-void string_clear(STRING **s);
-void string_dropfront(STRING **s, int nchars);
-void string_catvprintf(STRING **s, char *format, va_list ap);
-void string_catprintf(STRING **s, char *format, ...);
-void string_filterbadchars(STRING **s, int start);
-
 /* ftpsite.h */
 
 int ftp_site(FTPSTATE *peer, char *cmd);
@@ -530,14 +520,16 @@ void reallocwrapper(int size, void **inarea);
 char *strdupwrapper(char *s);
 void freewrapper(void *tofree);
 void freeifnotnull(void *);
+char *offt_tostr(off_t size);
+int strto_offt(char *str, off_t *ret);
 
 #ifndef HAVE_MEMMOVE
-char *memmove(void *dest, void *src, int n);
+void *memmove(void *, const void *, int);
 #endif
 
 void strtrimspace(char *string);
 int strchrcount(char *string, char tok);
-char *safe_vsnprintf(char *format, va_list ap);
+char *safe_vsnprintf(int size, char *format, va_list ap);
 char *safe_snprintf(char *format, ...);
 char *getcwd2(void);
 void pathname_simplify(char *pathname);
@@ -580,7 +572,7 @@ void unblockallsignals( void );
 
 void ftpd_setnogroups(void);
 void ftpd_preinit(void);
-void ftpd_init(char *filename) ;
+void ftpd_init(char *filename, int verbose);
 
 /* ftp.h */
 
@@ -725,7 +717,7 @@ VSERVER *findvserver(unsigned int ip, int port);
 void smartbind(SELECTER *selset, VSERVERCONN *list, int dodie);
 void dumbbind(SELECTER *selset, VSERVERCONN *list, int dodie);
 void sighandler(int signum);
-int mainprog(char *fconfig, int runforeground);
+int mainprog(char *fconfig, int runforeground, int verbose);
 void mainprog_inetd(char *fconfig, unsigned int ip);
 void diehandler(int signum);
 int main(int argc, char **argv);
@@ -733,7 +725,7 @@ int main(int argc, char **argv);
 /* datasock.h */
 
 int datasock_connect(SELECTER *sel, int fd, void *peerv);
-int startdatasocket(FTPSTATE *peer, int filefd, int trans_type, int tsize);
+int startdatasocket(FTPSTATE *peer, int filefd, int trans_type, off_t tsize);
 void closedatasocket(FTPSTATE *peer);
 void abortdatasocket(FTPSTATE *peer);
 
