@@ -17,7 +17,7 @@
 
 #include "ftpd.h"
 
-/* #define SHOWMALLOC */
+//#define SHOWMALLOC
 
 /* Wapper for malloc that syslogs and then dies if malloc fails before
    damage is done. */ 
@@ -126,18 +126,24 @@ void freeifnotnull(void *tofree)
 
 #ifndef HAVE_MEMMOVE
 
-char *memmove(void *dest, void *src, int n)
+void *memmove(void *dest, void *src, int n)
 {
-	void *tmparea = mallocwrapper(n);
+	int p;
 	
 	assert(dest != NULL);
 	assert(src != NULL);
 	assert(n >= 0);
 	
-	memcpy(tmparea, src, n);
-	memcpy(dest, tmparea, n);
-	freewrapper(tmparea);
+	if (src == dest)
+		return(dest);
 	
+	if (src < dest)
+		for(p = 0; p < n; p++)
+			*(char *)(src + n) = *(char *)(dest + n);
+	else
+		for(p = n; p > 0; p--)
+			*(char *)(src + n) = *(char *)(dest + n);	
+
 	return(dest);
 }
 
@@ -177,31 +183,33 @@ int strchrcount(char *string, char tok)
 	return(ret);
 }
 
-char *safe_vsnprintf(char *format, va_list ap)
+char *safe_vsnprintf(int size, char *format, va_list ap)
 {
-	int size = BUFFSMALL;
-	char *buffer = mallocwrapper(size);
+	char *buffer = mallocwrapper(size+1);
 	int result;
 	
-	result = vsnprintf(buffer, size, format, ap);
-	if (result >= size)
-	{
-		reallocwrapper(result+1, (void *)&buffer);
-		result = vsnprintf(buffer, result+1, format, ap);
-	}
-	
+	result = vsnprintf(buffer, size+1, format, ap);
 	return(buffer);
 }
 
 char *safe_snprintf(char *format, ...)
 {
 	va_list printfargs;
-	char *buf;
-	
+	int size = BUFFSMALL;
+	char *buf = mallocwrapper(size);
+	int result;
+
 	va_start(printfargs, format);
-	buf = safe_vsnprintf(format, printfargs);
+	result = vsnprintf(buf, size, format, printfargs);
 	va_end(printfargs);
 	
+	if (result >= size)
+	{
+		reallocwrapper(result+1, (void *)&buf);
+		va_start(printfargs, format);
+		result = vsnprintf(buf, result+1, format, printfargs);
+		va_end(printfargs);
+	}
 	return(buf);
 }
 
@@ -268,7 +276,7 @@ void pathname_simplify(char *pathname)
 				break;		
 			default:
 				pathname[pos2] = pathname[pos];
-				pos2++;
+			pos2++;
 		}
 		pos++;
 	}
@@ -281,6 +289,57 @@ void pathname_simplify(char *pathname)
 	
 	pathname[pos2] = 0;
 }	
+
+/* This is meant to be quick */
+
+char *offt_tostr(off_t size)
+{
+	/* the divide should be optimized away automagicly */
+	static char sizestr[sizeof(off_t) * 4];
+	char *l = sizestr + (sizeof(off_t) * 4 - 1);
+	off_t esize = size;
+	
+	*l = 0;
+	l--;
+	
+	do
+	{
+		*l = '0' + (esize % 10);
+		l--;
+		esize /= 10;
+	}
+	while(esize > 0);
+
+	if (size < 0)
+	{
+		*l = '-';
+		l--;
+	}
+	return(l + 1);
+}
+
+int strto_offt(char *str, off_t *ret)
+{
+	off_t r;
+	char *l = str;
+	int neg = (*l == '-');
+	
+	r = 0;
+	if (*l == '-')
+		l++;
+	
+	while((*l >= '0') && (*l <= '9'))
+	{
+		r = r * 10 + (*l - '0');
+		l++;
+	}
+	// error if not end of string
+	if (*l != 0)
+		return(-1);
+	
+	*ret = (neg ? -r : r);
+	return(0);
+}
 					
 void test_libc(int verbose)
 {
@@ -326,6 +385,9 @@ void test_libc(int verbose)
 		if (verbose) printf("No, ratio support disabled\n");
 		doesdint = FALSE;
 	}
+	
+	if ((!doesdint) && (sizeof(long long) == sizeof(off_t)))
+		ERRORMSGFATAL("snprintf doesn't support double int but needed to do file lengths");
 }
                                                 
 #ifndef HAVE_USLEEP
@@ -956,3 +1018,20 @@ void errormsgfatal( char *errmessage, char *file, int line )
 	exit(1);
 }
 
+/* 			close file descriptors 				*
+ ************************************************************************/
+
+// try to close all non-terminal file descriptors.
+
+void fd_closeall_nonterminal(void)
+{
+	int count, maxfilefd = 1024;
+#ifdef RLIMIT_NPROC
+	struct rlimit lim;
+	
+	getrlimit(RLIMIT_NOFILE, &lim);
+	maxfilefd = lim.rlim_max;
+#endif
+	for (count = 3; count < maxfilefd; count++)
+		close(count);
+}
