@@ -28,6 +28,6 @@ void showversion(char *desc)
 	printf(" version 2 or later. See http://www.fsf.org/copyleft/gpl.html\n");
 	printf(" for a copy of this license\n\n");
 	printf(" For updates to muddleftpd, visit its homesite at:\n");
-	printf(" http://www.muddleftpd.cx\n\n");
+	printf(" http://savannah.nongnu.org/projects/muddleftpd\n\n");
 	exit(1);
 }
